/*
	
					The Hacker's Diet
			http://www.fourmilab.ch/hackdiet/
	
	Weight and Exercise log for the Palm Platform
	
					by John Walker
				http://www.fourmilab.ch/
				
*/

#include "Hackdiet.h"

Boolean colourDevice = false;				// Does device support colour ?
Boolean useColour = false;					// Use colour where appropriate

//	determineFeatureSet  --  Set feature set flags based on ROM version

static Boolean atLeastROM2_0 = false;
Boolean featureSet2_0 = false;
Boolean featureSet3_0 = false;
Boolean featureSet3_1 = false;
Boolean featureSet3_2 = false;
Boolean featureSet3_5 = false;

static void determineFeatureSet(void)
{
	UInt32 romVersion;
	
	if (FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion) == 0) {
		if (romVersion >= 0x02000000) {
			atLeastROM2_0 = true;
			if (romVersion >= 0x02003000) {
				featureSet2_0 = true;
				if (romVersion >= 0x03003000) {
					featureSet3_0 = true;
					if (romVersion >= 0x03103000) {
						featureSet3_1 = true;
						if (romVersion >= 0x03203000) {
							featureSet3_2 = true;
							if (romVersion >= 0x03503000) {
								featureSet3_5 = true;
							}
						}
					}
				}
			}
		}
	}
}

//	StartApplication  --  Perform application-level initialisation when launched

static Err StartApplication(UInt16 cmd, UInt16 launchFlags)
{
	Boolean newdb;
	
//#define MEM_DEBUG
#ifdef MEM_DEBUG
	MemSetDebugMode(memDebugModeCheckOnChange | memDebugModeFillFree);
#endif
	if (featureSet3_5) {
		WinScreenMode(winScreenModeGetSupportsColor, NULL, NULL, NULL, &colourDevice);
	}
	
	getPreferences();				// Load preference settings
	newdb = hdEnsureDatabasePresent();

	/*	The following tippy-toes into the minefield of colour support.
		Above, we checked whether we're running on OS 3.5 or later.
		If not, "colour is not an option" and we must studiously
		avoid all system calls which presuppose its existence.  If
		we find ourselves on 3.5, we're entitled to call WinScreenMode
		to determine if the device is colour.  If it is, we set colourDevice,
		which permits the user to perform colour-related configuration
		tasks.  Finally, we interrogate the state of the pfNoColour flag
		in the application preferences.  Only if this is true do we
		proceed to the ultimate stage of setting useColour which actually
		enables colour coding in the various forms.  */
		
	if (colourDevice && (!PrefFlag(pfNoColour))) {
		useColour = true;
	}
				
	if (cmd != sysAppLaunchCmdGoTo) {
		
		if (!newdb) {
			DateTimeType dt;
			
			/*	If new database, defer creating the log for this
				month until after the user is done with the
				Preferences form so that it will be created in
				the chosen weight unit.  */
				
			TimSecondsToDateTime(TimGetSeconds(), &dt);
			hdEnsureMonthPresent(dt.year, dt.month - 1);
			logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
		}
		showToday = true;
		if (newdb) {
			FrmGotoForm(PreferencesForm);
		} else {
#ifdef CSV
#ifndef CSVDEBUG
			if (checkImportCSV()) {		// Import data from CSV database, if present
				FrmGotoForm(CSVImportForm);
				return 0;
			}	
#endif
#endif
			FrmGotoForm(MonthlyLogForm);
		}
	}
	
	return 0;
}

//	StopApplication  --  Perform application-level shutdown when exiting

static void StopApplication(void)
{
	FrmCloseAllForms();				// Close any open forms
	savePreferences();				// Save preference settings
}

/*	ApplicationHandleEvent  --  Application level event handler.
								Installs the form-level event handler
								when the user switches to a new form.  */

static Boolean ApplicationHandleEvent(EventPtr event)
{
	FormPtr	frm;
	Int16 formId;
	Boolean handled = false;

	if (event->eType == frmLoadEvent)
		{
		// Load the form resource specified in the event then activate the form.
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		/* Set the event handler for the form and initialise it.  The handler of the currently 
			active form is called by FrmDispatchEvent each time it receives an event. */
		switch (formId) {
#ifdef CSV
#ifdef CSVDEBUG
			case CSVDebugForm:
				csvDebugLoad(frm);
				break;
#endif
			case CSVImportForm:
				csvImportLoad(frm);
				break;
#endif

			case DietCalcForm:
				dietCalcLoad(frm);
				break;

			case ExerciseForm:
				exerciseLoad(frm);
				break;
				
			case HistoricalChartForm:
				historyLoad(frm);
				break;
				
			case MonthsForm:
				monthListLoad(frm);
				break;
				
			case MonthlyChartForm:
				chartingLoad(frm);
				break;
				
			case MonthlyLogForm:
				monthLogLoad(frm);
				break;
				
			case PreferencesForm:
				preferencesLoad(frm);
				break;
				
			case TrendAnalysisForm:
				trendanLoad(frm);
				break;
				
			case YearListForm:
				yearListLoad(frm);
				break;
				
			case YearViewForm:
				yearViewLoad(frm);
				break;
		}
		handled = true;
	}
	
	return handled;
}

//	Standard event loop: receive and dispatch events to hierarchical handlers

static void EventLoop(void)
{
	EventType event;
	UInt16 error;
	
	do {
		EvtGetEvent(&event, evtWaitForever);
		if (!SysHandleEvent(&event)) {
			if (!MenuHandleEvent(0, &event, &error)) {
				if (!ApplicationHandleEvent(&event)) {
					FrmDispatchEvent(&event);
				}
			}
		}
	}
	while (event.eType != appStopEvent);
}

//	Main program

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	Err err = 0;

#ifdef __GNUC__
		/*	If we're being called as a subroutine call, then 
			we need to do the prologue so that we can access our
			own globals. The system may be calling us for find.  */

	if (launchFlags & sysAppLaunchFlagSubCall) {
		CALLBACK_PROLOGUE
	}
#endif
	
	if (cmd == sysAppLaunchCmdNormalLaunch) {

		determineFeatureSet();				// Determine available features from ROM version
		
		/*	We require PalmOS 2.0 or later.  If we're running on a pre-2.0
			system post an alert and bail out.  */
			
		if (!atLeastROM2_0) {
			if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
				(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) {
				FrmAlert(SystemIncompatibleAlert);
				
				/*	PalmOS 1.0 launches a failed application over and over again
					unless we explicitly switch to another.  Make it so.  */
					
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				return sysErrRomIncompatible;
			}
		}
		if ((err = StartApplication(cmd, launchFlags)) == 0) {	// Initialise
			EventLoop();							// Process events while in control
			StopApplication();						// Clean up when done
		}
	} else if (cmd == sysAppLaunchCmdSaveData) {
		FrmSaveAllForms();
#ifdef FIND_SUPPORT
	} else if (cmd == sysAppLaunchCmdFind) {
		findQuery((FindParamsPtr) cmdPBP);
	} else if (cmd == sysAppLaunchCmdGoTo) {
		Boolean	launched;
		launched = launchFlags & sysAppLaunchFlagNewGlobals;
		
		if (launched) {
			err = StartApplication(cmd, launchFlags);
			if (!err) {
				findGoTo((GoToParamsPtr) cmdPBP, launched);
				EventLoop();
				StopApplication();	
			} else {
				return err;
			}
		} else {
#ifdef __GNUC__
			CALLBACK_PROLOGUE
#endif
			findGoTo((GoToParamsPtr) cmdPBP, launched);
#ifdef __GNUC__
			CALLBACK_EPILOGUE
#endif
			return err;
		}

	
#ifdef __GNUC__
	if (launchFlags & sysAppLaunchFlagSubCall) {
		CALLBACK_EPILOGUE
	}
#endif
#endif
	}
	return err;
}
